@@ -53,11 +53,11 @@ def autorotate_image(img, corners):
     high_corners = (numpy.argmin(corners_y), numpy.argmax(corners_y),
                     numpy.argmin(corners_x), numpy.argmax(corners_x))
 
-    if high_corners == (0, 1, 3, 1) or high_corners == (3, 1, 2, 1):    # 90 degrees
+    if high_corners == (0, 2, 3, 1):    # 90 degrees
         return 90, numpy.rot90(img, 1)
-    elif high_corners == (3, 0, 1, 0):  # 180 degrees
+    elif high_corners == (3, 1, 1, 3) or high_corners == (3, 1, 2, 0):  # 180 degrees
         return 180, cv2.flip(img, -1)
-    elif high_corners == (1, 3, 0, 2):  # 270 degrees
+    elif high_corners == (2, 0, 1, 3):  # 270 degrees
         return 270, numpy.rot90(img, 3)
     else:
         # Do nothing for zero-degree rotations
@@ -114,7 +114,8 @@ def get_scaled_corners(thumbnail_image, source_image, full_source_image, kp_pair
     return corners
 
 
-def reconstruct_thumbnail(thumbnail_image, source_image, corners, downsize_reconstruction=False):
+def reconstruct_thumbnail(thumbnail_image, source_image, corners, downsize_reconstruction=False,
+                          max_aspect_ratio_delta=0.1):
     logging.info("Reconstructing thumbnail from source image")
 
     thumb_h, thumb_w = thumbnail_image.shape[:2]
@@ -133,13 +134,18 @@ def reconstruct_thumbnail(thumbnail_image, source_image, corners, downsize_recon
     if downsize_reconstruction and (new_thumb_h > thumb_h or new_thumb_w > thumb_w):
         new_thumb = fit_image_within(new_thumb, thumb_h, thumb_w)
 
+    old_aspect_ratio = thumbnail_image.shape[0] / thumbnail_image.shape[1]
+    new_aspect_ratio = new_thumb.shape[0] / new_thumb.shape[1]
     logging.info('Master dimensions: width=%s, height=%s', source_image.shape[1], source_image.shape[0])
     logging.info('Thumbnail dimensions: width=%s, height=%s (aspect ratio: %0.3f)',
                  thumbnail_image.shape[1], thumbnail_image.shape[0],
-                 thumbnail_image.shape[0] / thumbnail_image.shape[1])
+                 old_aspect_ratio)
     logging.info('Reconstructed thumb dimensions: width=%s, height=%s (rotation=%d°, aspect ratio: %0.3f)',
                  new_thumb.shape[1], new_thumb.shape[0],
-                 new_thumb_rotation, new_thumb.shape[0] / new_thumb.shape[1])
+                 new_thumb_rotation, new_aspect_ratio)
+
+    if abs(old_aspect_ratio - new_aspect_ratio) > max_aspect_ratio_delta:
+        raise RuntimeError('Aspect ratios are significantly different – reconstruction likely failed!')
 
     return new_thumb, new_thumb_crop, new_thumb_rotation
 
@@ -205,6 +211,7 @@ def find_homography(kp_pairs):
 
 def locate_thumbnail(thumbnail_filename, source_filename, display=False, save_visualization=False,
                      save_reconstruction=False, reconstruction_format="jpg",
+                     max_aspect_ratio_delta=0.1, minimum_matches=10,
                      json_output_filename=None, max_master_edge=4096, max_output_edge=2048):
     thumbnail_basename, thumbnail_image = open_image(thumbnail_filename)
     source_basename, source_image = open_image(source_filename)
@@ -216,9 +223,10 @@ def locate_thumbnail(thumbnail_filename, source_filename, display=False, save_vi
         logging.info("Resizing master to fit within %d pixels", max_master_edge)
         source_image = fit_image_within(source_image, max_master_edge, max_master_edge)
 
+    logging.info('Finding common features')
     kp_pairs = match_images(thumbnail_image, source_image)
 
-    if len(kp_pairs) >= 4:
+    if len(kp_pairs) >= minimum_matches:
         title = "Found %d matches" % len(kp_pairs)
         logging.info(title)
 
@@ -226,7 +234,8 @@ def locate_thumbnail(thumbnail_filename, source_filename, display=False, save_vi
 
         corners = get_scaled_corners(thumbnail_image, source_image, full_source_image, kp_pairs, H)
 
-        new_thumbnail, corners, rotation = reconstruct_thumbnail(thumbnail_image, full_source_image, corners)
+        new_thumbnail, corners, rotation = reconstruct_thumbnail(thumbnail_image, full_source_image, corners,
+                                                                 max_aspect_ratio_delta=max_aspect_ratio_delta)
 
         if json_output_filename:
             with open(json_output_filename, mode='wb') as json_file:
@@ -299,6 +308,11 @@ def main():
                              "(faster but possibly less accurate)")
     parser.add_argument('--fit-output-within', type=int, default=2048,
                         help="Resize output so the largest edge is below the specified value")
+    parser.add_argument('--minimum-matches', type=int, default=20,
+                        help='Require at least this many features for a match (default %(default)s)')
+    parser.add_argument('--max-aspect-ratio-delta', type=float, default=0.1,
+                        help='Raise an error if the reconstructed image\'s aspect ratio differs by more than '
+                             'this percentage default %(default)s)')
     parser.add_argument('--display', action="store_true", help="Display match visualization")
     parser.add_argument('--debug', action="store_true", help="Open debugger for errors")
     args = parser.parse_args()
@@ -328,7 +342,9 @@ def main():
                              save_visualization=args.save_visualization,
                              json_output_filename=json_output_filename,
                              max_master_edge=args.fit_master_within,
-                             max_output_edge=args.fit_output_within)
+                             max_output_edge=args.fit_output_within,
+                             max_aspect_ratio_delta=args.max_aspect_ratio_delta,
+                             minimum_matches=args.minimum_matches)
         except Exception as e:
             logging.error("Error processing %s %s: %s", thumbnail, source, e)
             if args.debug:
